var parentItem;
var canvas_width
var canvas_height;
var center_x;
var stretch_factor;
var y_speed;
var t;
var ctx;


var Vector = function (x, y) {
    this.x = x || 0;
    this.y = y || 0;
};

Vector.prototype = {
    add: function (v) {
        this.x += v.x;
        this.y += v.y;
        return this;
    },
    length: function () {
        return Math.sqrt(this.x * this.x + this.y * this.y);
    },
    rotate: function (theta) {
        var x = this.x;
        var y = this.y;
        this.x = Math.cos(theta) * this.x - Math.sin(theta) * this.y;
        this.y = Math.sin(theta) * this.x + Math.cos(theta) * this.y;
        //this.x = Math.cos(theta) * x - Math.sin(theta) * y;
        //this.y = Math.sin(theta) * x + Math.cos(theta) * y;
        return this;
    },
    mult: function (f) {
        this.x *= f;
        this.y *= f;
        return this;
    }
};

var Leaf = function (p, r, c, ctx) {
    this.p = p || null;
    this.r = r || 0;
    this.color = Branch.randomrgba(0,255,.1);//c || 'rgba(255,255,255,1.0)';
    this.ctx = ctx;
}

Leaf.prototype = {
    render: function () {
        var that = this;
        var ctx = this.ctx;
        var f = Branch.random(1, 2)
        for (var i = 0; i < 5; i++) {
            (function (r) {
                QExtend.setTimeout(function () {
                    ctx.beginPath();
                    ctx.fillStyle = that.color;
                    ctx.moveTo(that.p.x, that.p.y);
                    ctx.arc(that.p.x, that.p.y, r, 0, Branch.circle, true);
                    ctx.fill();
                }, r * 60);
            })(i);
        }
    }
}


var Branch = function (p, v, r, c, t) {
    this.p = p || null;
    this.v = v || null;
    this.r = r || 0;
    this.length = 0;
    this.generation = 1;
    this.tree = t || null;
    this.color = c || 'rgba(255,255,255,1.0)';
    this.register();
};

Branch.prototype = {
    register: function () {
        this.tree.addBranch(this);
    },
    draw: function () {
        var ctx = this.tree.ctx;
        ctx.beginPath();
        ctx.fillStyle = this.color;
        ctx.moveTo(this.p.x, this.p.y);
        ctx.arc(this.p.x, this.p.y, this.r, 0, Branch.circle, true);
        ctx.fill();
    },
    modify: function () {
        var angle = 0.18 - (0.10 / this.generation);
        this.p.add(this.v);
        this.length += this.v.length();
        this.r *= 0.99;
        this.v.rotate(Branch.random(-angle, angle)); //.mult(0.996);
        if (this.r < 0.8 || this.generation > 10) {
            this.tree.removeBranch(this);
            var l = new Leaf(this.p, 10, this.color, this.tree.ctx);
            l.render();
        }
    },
    grow: function () {
        this.draw();
        this.modify();
        this.fork();
    },
    fork: function () {
        var p = this.length - Branch.random(100, 200); // + (this.generation * 10);
        if (p > 0) {
            var n = Math.round(Branch.random(1, 3));
            this.tree.stat.fork += n - 1;
            for (var i = 0; i < n; i++) {
                Branch.clone(this);
            }
            this.tree.removeBranch(this);
        }
    }
};

Branch.circle = 2 * Math.PI;
Branch.random = function (min, max) {
    return Math.random() * (max - min) + min;
};
Branch.clone = function (b) {
    var r = new Branch(new Vector(b.p.x, b.p.y), new Vector(b.v.x, b.v.y), b.r, b.color, b.tree);
    r.generation = b.generation + 1;
    return r;
};
Branch.rgba = function (r, g, b, a) {
    return 'rgba(' + r + ',' + g + ',' + b + ',' + a + ')';
};
Branch.randomrgba = function (min, max, a) {
    return Branch.rgba(Math.round(Branch.random(min, max)), Math.round(Branch.random(min, max)), Math.round(Branch.random(min, max)), a);
};

var Tree = function () {
    var branches = [];
    var timer;
    this.stat = {
        fork: 0,
        length: 0
    };
    this.addBranch = function (b) {
        branches.push(b);
    };
    this.removeBranch = function (b) {
        for (var i = 0; i < branches.length; i++) {
            if (branches[i] === b) {
                branches.splice(i, 1);
                return;
            }
        }
    };
    this.render = function () {
        var that = this;
        timer = QExtend.setInterval(function () {
            if (branches.length > 0) {
                for (var i = 0; i < branches.length; i++) {
                    branches[i].grow();
                }
            }
            else {
                //clearInterval(timer);
            }
        }, 1000 / 30);
    };
    this.init = function (ctx) {
        this.ctx = ctx;
    };
    this.abort = function () {
        branches = [];
        this.stat = {
            fork: 0,
            length: 0
        }
    };
};



function setup(ct,width,height) {
    ctx = ct;
    canvas_width = width;
    canvas_height = height;
    center_x = canvas_width / 2;
    stretch_factor = 600 / canvas_height;
    y_speed = 3 / stretch_factor;
    QExtend.setTimeout(function() {
        ctx.globalCompositeOperation = "source-over"; // switch to additive color application
        {
            ctx.fillStyle = "rgb(0,0,0)";
            ctx.fillRect(0, 0, canvas_width,canvas_height); // resize <canvas> and draw black rect (default)
        }
        ctx.globalCompositeOperation = "lighter";},1);
    t = new Tree();
    t.init(ctx);
    t.render();
    // init


}





function onkeydown(mouseX,mouseY)
{
    for (var i = 0; i < 3; i++) {
        new Branch(new Vector(mouseX, canvas_height), new Vector(0, -y_speed), 15 / stretch_factor, Branch.randomrgba(0, 255, .1), t);
    }
}
// events

/*  $("#drawArea").click(function (e) {
//e.preventDefault();
                        var x, y;
                        x = e.pageX - this.offsetLeft;
                        y = e.pageY - this.offsetTop;
                        new Branch(new Vector(x, canvas_height), new Vector(0, -y_speed), 15 / stretch_factor, Branch.randomrgba(0, 255, 0.3), t);
                    });
                   /* $("#btnClear").click(function (e) {
                        e.stopPropagation();
                        t.abort();
                        ctx.clearRect(0, 0, canvas_width, canvas_height);
                        $statMsg.html("0");
                    });
                    $("#btnReload").click(function (e) {
                        e.stopPropagation();
                        window.location.reload();
                    });
                    $("#btnNewExperiment").click(function (e) {
                        window.location = "http://www.openrise.com/lab/FlowerPower";
                    });*/
